@@ -22,7 +22,11 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
  */
 
-//! # zarray
+//! # ZArray
+//! Z-order indexed 2D and 3D arrays using Morton order (aka Z-order) with a convenient API for
+//! common 2D and 3D access patterns. Use of zarray in place of a Vec of Vecs often improves
+//! performance, especially for algorithms such as blurring and cellular automata.
+//! ## About ZArray
 //! The *zarray* crate  is a lightweight Rust library that provides structs for working with 2D and
 //! 3D arrays, using internal Z-Order Morton indexing to improve data localization for better
 //! cache-line performance.
@@ -30,11 +34,11 @@ SOFTWARE.
 //! Simply import *zarray::z2d::ZArray2D* and/or *zarray::z3d::ZArray3D* and then use the
 //! *ZArray_D::new(...)* function to initialize a new instance. The type will automatically be
 //! inferred from the povided default value. Note that only types which implement the *Copy* trait
-//! are allowed (ie not Vec or other head-allocating types).
+//! are allowed (ie not Vec or other heap-allocating types).
 //!
-//! For example, here's a simple blur operation using ZArray2D, which generally outperforms better
+//! For example, here's a simple blur operation using ZArray2D, which generally performs better
 //! than using a Vec of Vecs by about 10-25%:
-//! ```
+//! ```rust
 //! use zarray::z2d::ZArray2D;
 //! let h: isize = 200;
 //! let w: isize = 300;
@@ -55,7 +59,7 @@ SOFTWARE.
 //!
 //! ## How it works
 //! the *ZArray_D* structs store data in 8x8 or 8x8x8 chuncks, using Z-order indexing to access the
-//! data within each chunk (as described [here](https://en.wikipedia.org/wiki/Z-order_curve) ). In0
+//! data within each chunk (as described [here](https://en.wikipedia.org/wiki/Z-order_curve) ). In
 //! so doing, the lowest 4 bits of each dimension are interdigitated to significantly improve data
 //! locality and cache-line fetch efficiency (though not as much as a Hilbert curve would do)
 //!
@@ -63,7 +67,7 @@ SOFTWARE.
 //! Most of the time, using a Vec<Vec<T>> would have great performance, so long as you remember to
 //! structure your for-loops correctly. However, when the data is not accessed in a linear fashion,
 //! such as when implementing a cellular automata or a blurring or ray tracing algorithm, then the
-//! performance of a Vec<Vec<T>> and be significantly impaired by frequent RAM access and
+//! performance of a Vec<Vec<T>> can be significantly impaired by frequent RAM access and
 //! cache-line misses. This is when data locality matters most for performance.
 //!
 //! ### Why not Z-Order the entire data array?
@@ -74,7 +78,7 @@ SOFTWARE.
 //! addressing space (ie 8x8 or 4x4x4)
 //!
 //! ## Note
-//! Only types with the *Copy* trait can be stored in *ZArrady_D* structs. Thus *zarray* works for
+//! Only types with the *Copy* trait can be stored in *ZArray_D* structs. Thus *zarray* works for
 //! all numerical types and most simple data structs, but not for heap-allocating data types such
 //! as Vec. This limitation arises from complexities in filling the data patches with initial
 //! values. In addition, you would not see a performance improvement over a simple Vec<Vec<T>> if
@@ -84,8 +88,7 @@ SOFTWARE.
 //! This library is provided under the MIT license. In other words: free to use as you wish.
 //!
 //! ## Contributing
-//! If you'd like to contribute, go ahead and fork the GitHub repo and/or submit a pull request:
-//! [ZArray-rs GitHub page](https://github.com/DrPlantabyte/ZArray-rs/)
+//! If you'd like to contribute, go ahead and fork the GitHub repo and/or submit a pull request
 
 use std::error::Error;
 use std::fmt::{Debug, Display, Formatter};
@@ -247,6 +250,42 @@ pub mod z2d {
 			return ZArray2D {width, height, pwidth, patches: p, _phantomdata: PhantomData};
 		}
 
+		/// Gets the (x, y) size of this 2D array
+		/// # Returns
+		/// Returns a tuple of (width, height) for this 2D array
+		pub fn dimensions(&self) -> (usize, usize){
+			return (self.width, self.height);
+		}
+
+		/// Gets the X-dimension size (aka width) of this 2D array
+		/// # Returns
+		/// Returns the size in the X dimension
+		pub fn xsize(&self) -> usize {
+			return self.width;
+		}
+
+
+		/// Alias for `xsize()`
+		/// # Returns
+		/// Returns the size in the X dimension
+		pub fn width(&self) -> usize {
+			return self.xsize();
+		}
+
+		/// Gets the Y-dimension size (aka height) of this 2D array
+		/// # Returns
+		/// Returns the size in the Y dimension
+		pub fn ysize(&self) -> usize {
+			return self.height;
+		}
+
+		/// Alias for `ysize()`
+		/// # Returns
+		/// Returns the size in the Y dimension
+		pub fn height(&self) -> usize {
+			return self.ysize();
+		}
+
 		/// Gets a value from the 2D array, or returns a *LookUpError* if the provided coordinate
 		/// is out of bounds. If you are using a default value for out-of-bounds coordinates,
 		/// then you should use the *bounded_get(x, y)* method instead. If you want access to
@@ -388,21 +427,6 @@ pub mod z2d {
 			Ok(())
 		}
 
-		/// Fills a region of this 2D array with a given value, ignoring any
-		/// coordinates that go out of bounds.
-		/// # Parameters
-		/// * **x1** - the first x dimension coordinate (inclusive)
-		/// * **y1** - the first y dimension coordinate (inclusive)
-		/// * **x2** - the second x dimension coordinate (exclusive)
-		/// * **y2** - the second y dimension coordinate (exclusive)
-		/// * **new_val** - value to store in the 2D array in the bounding box defined by
-		/// (x1, y1) -> (x2, y2)
-		pub fn wrapped_fill(&mut self, x1: isize, y1: isize, x2: isize, y2: isize, new_val: T) {
-			for y in y1..y2{ for x in x1..x2{
-				self.wrapped_set(x, y, new_val);
-			} }
-		}
-
 		/// Fills a region of this 2D array with a given value, wrapping the axese when
 		/// coordinates go out of bounds.
 		/// # Parameters
@@ -412,6 +436,21 @@ pub mod z2d {
 		/// * **y2** - the second y dimension coordinate (exclusive)
 		/// * **new_val** - value to store in the 2D array in the bounding box defined by
 		/// (x1, y1) -> (x2, y2) with wrapped axese
+		pub fn wrapped_fill(&mut self, x1: isize, y1: isize, x2: isize, y2: isize, new_val: T) {
+			for y in y1..y2{ for x in x1..x2{
+				self.wrapped_set(x, y, new_val);
+			} }
+		}
+
+		/// Fills a region of this 2D array with a given value, ignoring any
+		/// coordinates that go out of bounds.
+		/// # Parameters
+		/// * **x1** - the first x dimension coordinate (inclusive)
+		/// * **y1** - the first y dimension coordinate (inclusive)
+		/// * **x2** - the second x dimension coordinate (exclusive)
+		/// * **y2** - the second y dimension coordinate (exclusive)
+		/// * **new_val** - value to store in the 2D array in the bounding box defined by
+		/// (x1, y1) -> (x2, y2)
 		pub fn bounded_fill(&mut self, x1: isize, y1: isize, x2: isize, y2: isize, new_val: T) {
 			for y in y1..y2{ for x in x1..x2{
 				self.bounded_set(x, y, new_val);
@@ -482,8 +521,8 @@ pub mod z2d {
 
 /// This module is used for storing 3-dimensional data arrays, and internally uses Z-index arrays
 /// to improve data localization and alignment to the CPU cache-line fetches. In other words, use
-/// this to improve performance for 2D data that is randomly accessed rather than raster scanned
-/// or if your data processing makes heavy use of neighbor look-up in both the X and Y directions.
+/// this to improve performance for 3D data that is randomly accessed rather than raster scanned
+/// or if your data processing makes heavy use of neighbor look-up in the X, Y, and Z directions.
 /// # How It Works
 /// When you initialize a zarray::z3d::ZArray3D struct, it creates an array of 8x8x8 data patches
 /// (512 total elements per patch), using Z-curve indexing within that patch. When you call a
@@ -610,6 +649,56 @@ pub mod z3d {
 				patches: p, _phantomdata: PhantomData};
 		}
 
+		/// Gets the (x, y, z) size of this 3D array
+		/// # Returns
+		/// Returns a tuple of (width, height, depth) for this 2D array
+		pub fn dimensions(&self) -> (usize, usize, usize){
+			return (self.xsize, self.ysize, self.zsize);
+		}
+
+		/// Gets the X-dimension size (aka width) of this 3D array
+		/// # Returns
+		/// Returns the size in the X dimension
+		pub fn xsize(&self) -> usize {
+			return self.xsize;
+		}
+
+
+		/// Alias for `xsize()`
+		/// # Returns
+		/// Returns the size in the X dimension
+		pub fn width(&self) -> usize {
+			return self.xsize();
+		}
+
+		/// Gets the Y-dimension size (aka height) of this 3D array
+		/// # Returns
+		/// Returns the size in the Y dimension
+		pub fn ysize(&self) -> usize {
+			return self.ysize;
+		}
+
+		/// Alias for `ysize()`
+		/// # Returns
+		/// Returns the size in the Y dimension
+		pub fn height(&self) -> usize {
+			return self.ysize();
+		}
+
+		/// Gets the Z-dimension size (aka depth) of this 3D array
+		/// # Returns
+		/// Returns the size in the Z dimension
+		pub fn zsize(&self) -> usize {
+			return self.zsize;
+		}
+
+		/// Alias for `zsize()`
+		/// # Returns
+		/// Returns the size in the Z dimension
+		pub fn depth(&self) -> usize {
+			return self.zsize();
+		}
+
 
 		/// Gets a value from the 3D array, or returns a *LookUpError* if the provided coordinate
 		/// is out of bounds. If you are using a default value for out-of-bounds coordinates,
@@ -775,8 +864,8 @@ pub mod z3d {
 			Ok(())
 		}
 
-		/// Fills a region of this 3D array with a given value, ignoring any
-		/// coordinates that go out of bounds.
+		/// Fills a region of this 3D array with a given value, wrapping the axese when
+		/// coordinates go out of bounds.
 		/// # Parameters
 		/// * **x1** - the first x dimension coordinate (inclusive)
 		/// * **y1** - the first y dimension coordinate (inclusive)
@@ -793,8 +882,8 @@ pub mod z3d {
 			} } }
 		}
 
-		/// Fills a region of this 3D array with a given value, wrapping the axese when
-		/// coordinates go out of bounds.
+		/// Fills a region of this 3D array with a given value, ignoring any
+		/// coordinates that go out of bounds.
 		/// # Parameters
 		/// * **x1** - the first x dimension coordinate (inclusive)
 		/// * **y1** - the first y dimension coordinate (inclusive)
@@ -885,6 +974,13 @@ mod tests {
 		let w: usize = 809;
 		let (mut ref_map, mut map) = seed_arrays_u8(w, h);
 		let mut prng = StdRng::seed_from_u64(20220331u64);
+		// assert get sizes
+		assert_eq!(map.dimensions().0, w);
+		assert_eq!(map.dimensions().1, h);
+		assert_eq!(map.xsize(), w);
+		assert_eq!(map.width(), w);
+		assert_eq!(map.ysize(), h);
+		assert_eq!(map.height(), h);
 		// set values
 		for y in 0..h {
 			for x in 0..w {
@@ -1138,6 +1234,16 @@ mod tests {
 		let d: usize = 23;
 		let (mut ref_map, mut map) = seed_3darrays_u8(w, h, d);
 		let mut prng = StdRng::seed_from_u64(20220331u64);
+		// assert get sizes
+		assert_eq!(map.dimensions().0, w);
+		assert_eq!(map.dimensions().1, h);
+		assert_eq!(map.dimensions().2, d);
+		assert_eq!(map.xsize(), w);
+		assert_eq!(map.width(), w);
+		assert_eq!(map.ysize(), h);
+		assert_eq!(map.height(), h);
+		assert_eq!(map.zsize(), d);
+		assert_eq!(map.depth(), d);
 		// set values
 		for z in 0..d {
 			for y in 0..h {
